@@ -1,6 +1,6 @@
 /* deflate.c -- compress data using the deflation algorithm
 
-   Copyright (C) 1999, 2006, 2009-2021 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2006, 2009-2022 Free Software Foundation, Inc.
    Copyright (C) 1992-1993 Jean-loup Gailly
 
    This program is free software; you can redistribute it and/or modify
@@ -300,9 +300,10 @@ local  void check_match (IPos start, IPos match, int length);
 
 /* ===========================================================================
  * Initialize the "longest match" routines for a new file
+ * PACK_LEVEL values: 0: store, 1: best speed, 9: best compression
  */
-void lm_init (pack_level)
-    int pack_level; /* 0: store, 1: best speed, 9: best compression */
+static void
+lm_init (int pack_level)
 {
     register unsigned j;
 
