@@ -1,5 +1,6 @@
 #include <config.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* Trivial example of reading a gzip'ed file or gzip'ed standard input
  * using stdio functions fread(), getc(), etc... fseek() is not supported.
@@ -19,36 +20,36 @@ int main(argc, argv)
     int n;
 
     if (argc < 1 || argc > 2) {
-	fprintf(stderr, "usage: %s [file[.gz]]\n", argv[0]);
-	exit(1);
+        fprintf(stderr, "usage: %s [file[.gz]]\n", argv[0]);
+        exit(EXIT_FAILURE);
     }
     strcpy(cmd, "gzip -dc ");  /* use "gzip -c" for zwrite */
     if (argc == 2) {
-	strncat(cmd, argv[1], sizeof(cmd)-strlen(cmd));
+        strncat(cmd, argv[1], sizeof(cmd)-strlen(cmd));
     }
     infile = popen(cmd, "r");  /* use "w" for zwrite */
     if (infile == NULL) {
-	fprintf(stderr, "%s: popen('%s', 'r') failed\n", argv[0], cmd);
-	exit(1);
+        fprintf(stderr, "%s: popen('%s', 'r') failed\n", argv[0], cmd);
+        exit(EXIT_FAILURE);
     }
     /* Read one byte using getc: */
     n = getc(infile);
     if (n == EOF) {
-	pclose(infile);
-	exit(0);
+        pclose(infile);
+        exit(EXIT_SUCCESS);
     }
     putchar(n);
 
     /* Read the rest using fread: */
     for (;;) {
-	n = fread(buf, 1, BUFSIZ, infile);
-	if (n <= 0) break;
-	fwrite(buf, 1, n, stdout);
+        n = fread(buf, 1, BUFSIZ, infile);
+        if (n <= 0) break;
+        fwrite(buf, 1, n, stdout);
     }
     if (pclose(infile) != 0) {
-	fprintf(stderr, "%s: pclose failed\n", argv[0]);
-	exit(1);
+        fprintf(stderr, "%s: pclose failed\n", argv[0]);
+        exit(EXIT_FAILURE);
     }
-    exit(0);
+    exit(EXIT_SUCCESS);
     return 0; /* just to make compiler happy */
 }
